# Generated by Django 3.0.7 on 2020-07-21 08:50

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Email',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(max_length=255)),
                ('mail_to', models.EmailField(max_length=254)),
                ('cc', models.TextField(null=True)),
                ('bcc', models.TextField(null=True)),
                ('body', models.TextField()),
                ('html_body', models.TextField()),
                ('attachment', models.FileField(null=True, upload_to='invoices/%Y/%m/%d')),
                ('start_send_at', models.DateTimeField(blank=True, null=True)),
                ('sent_at', models.DateTimeField(blank=True, null=True)),
            ],
        ),
    ]



# Generated by Django 4.0.2 on 2022-02-24 16:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mailing', '0002_auto_20200722_1834'),
    ]

    operations = [
        migrations.AddField(
            model_name='email',
            name='email_from',
            field=models.EmailField(default='nonexistent@example.com', max_length=254, verbose_name='Mail From'),
            preserve_default=False,
        ),
    ]



import os
from typing import List, Union

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.utils import timezone as tz


def send_mail(email) -> EmailMultiAlternatives:
    """Send an email message."""
    email = process_whitelist(email)
    if email.sent_at is not None:
        return EmailMultiAlternatives('Blocked', 'Blocked')

    email.start_send_at = tz.now()
    email.save()

    email_message = EmailMultiAlternatives(
        email.subject, email.body, email.email_from, [email.mail_to])
    email_message.attach_alternative(email.html_body, "text/html")
    if email.cc:
        email_message.cc = email.cc.split()
    if email.bcc:
        email_message.bcc = email.bcc.split()
    if email.attachment:
        name = os.path.basename(email.attachment.url)
        email_message.attach(name, email.attachment.read())

    email_message.send()

    email.sent_at = tz.now()
    email.save()

    return email_message


def _is_email_whitelisted(addr: Union[str, None]) -> bool:
    """
    Check if email is contained within an EMAIL_WHITELIST setting.

    It will attempt to match an exact address or same domain match
    if the whitelisted address is listed as `*@domain.com`.
    """
    whitelist: List[str] = settings.EMAIL_WHITELIST
    if not addr:
        return False
    elif addr in whitelist:
        return True

    whitelisted_by_domain = False
    for e in whitelist:
        if not e.startswith('*'):
            continue
        _, whitelisted_domain = e.split('@')
        _, domain = addr.split('@')
        if domain == whitelisted_domain:
            whitelisted_by_domain = True
            break

    return whitelisted_by_domain


def process_whitelist(email):
    """
    Compare all recipients against the EMAIL_WHITELIST setting and alter instance if needed.

    Checks for each of mail_to, cc and bcc addresses
    - If the whitelist is empty or each recipient is whitelisted does not change any email
    - If the whitelist is not empty, appends [TEST EMAIL] to subject
    - If no recipient is valid, replaces `@` with `AT` for each recipient
        and marks the Email as sent.
    - If any recipient is valid, sends to all valid recipients.
    - If `mail_to` is not valid, sets a valid `cc` or `bcc` as `mail_to` .
    """
    whitelist_empty = len(settings.EMAIL_WHITELIST) == 0
    if email.sent_at or whitelist_empty:
        return email

    # Add TEST EMAIL prefix if sent from an environment that restricts recipients
    if not whitelist_empty:
        email.subject = f'[TEST EMAIL] {email.subject}'

    to_whitelisted = _is_email_whitelisted(email.mail_to)
    cc_whitelisted = _is_email_whitelisted(email.cc)
    bcc_whitelisted = _is_email_whitelisted(email.bcc)

    if not (to_whitelisted or cc_whitelisted or bcc_whitelisted):
        email.mail_to = email.mail_to.replace('@', 'AT')
        email.cc = email.cc and email.cc.replace('@', 'AT')
        email.bcc = email.bcc and email.bcc.replace('@', 'AT')
        email.start_send_at = tz.now()
        email.sent_at = tz.now()
        email.save()
        return email

    if not cc_whitelisted:
        email.cc = None
    if not bcc_whitelisted:
        email.bcc = None
    # to_email can't be null, so set it to any of the whitelisted
    if not to_whitelisted:
        email.mail_to = email.cc or email.bcc

    email.save()
    return email



from django.db import models
from django.utils.translation import gettext_lazy as _

from mailing.conf import settings
from mailing.tasks import task_delay_robust


class EmailManager(models.Manager):
    def queue_email(
        self, subject, body, mail_to, html_body, cc=None, bcc=None,
        attachment=None, auto_send=True, email_from=settings.MAILING_EMAIL_FROM,
        taskargs=[]
    ):
        email = self.create(
            subject=subject, body=body, email_from=email_from,
            mail_to=mail_to, html_body=html_body, cc=cc, bcc=bcc, attachment=attachment)
        if auto_send:
            # If this code is running within transaction.atomic block, it should wait for
            # the txn to commit before delaying the task, otherwise inserted email may not be
            # available to the celery worker when it tries to access it.
            task_delay_robust(settings.MAILING_SEND_TASK, email.pk, *taskargs)
        return email


class Email(models.Model):
    subject = models.CharField(max_length=255, verbose_name=_('Subject'))
    mail_to = models.EmailField(verbose_name=_('Mail To'))
    cc = models.TextField(null=True, verbose_name=_('cc'))
    bcc = models.TextField(null=True, verbose_name=_('bcc'))
    email_from = models.EmailField(verbose_name=_('Mail From'))
    body = models.TextField(verbose_name=_('Body'))
    html_body = models.TextField(verbose_name=_('HTML Body'))
    attachment = models.FileField(
        upload_to='invoices/%Y/%m/%d',
        null=True,
        verbose_name=_('Attachment'))
    start_send_at = models.DateTimeField(null=True, blank=True, verbose_name=_('Start Send At'))
    sent_at = models.DateTimeField(null=True, blank=True, verbose_name=_('Sent At'))

    objects = EmailManager()



from celery import Celery

INSTALLED_APPS = [
    'mailing',
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'mailing',
    }
}

MAILING_EMAIL_FROM = 'mailing@example.com'

CELERY_TASK_ALWAYS_EAGER = True

app = Celery('mailing')

app.config_from_object('django.conf:settings', namespace='CELERY')
app.autodiscover_tasks()

print(app.conf.__dict__.keys())



import factory.fuzzy
import factory

from mailing.models import Email


class EmailFactory(factory.django.DjangoModelFactory):
    subject = factory.fuzzy.FuzzyText(length=50)
    body = factory.fuzzy.FuzzyText(length=255)
    html_body = factory.fuzzy.FuzzyText(length=255)

    class Meta:
        model = Email



from celery import shared_task

import logging

from django.conf import settings
from django.db import transaction

from mailing.services import send_mail

log = logging.getLogger(__name__)


@shared_task
def send_mail_task(email_id):
    """Celery task to send emails."""
    from mailing.models import Email
    task_id = send_mail_task.request.id
    mail = Email.objects.get(pk=email_id)
    try:
        send_mail(mail)
    except Exception:
        log.exception("Failed to send email")
    return {'task_id': task_id}


def task_delay_robust(task, *args):
    """Run celery task keeping db transaction and eager execution in mind."""
    def send_delayed():
        return task.delay(*args)

    delay_on_tx_commit = (
        getattr(settings, 'CELERY_TASK_ALWAYS_EAGER', False) is False and
        transaction.get_connection().in_atomic_block
    )
    if delay_on_tx_commit:
        transaction.on_commit(send_delayed)
    else:
        return send_delayed()



# Generated by Django 3.0.7 on 2020-07-22 18:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mailing', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='email',
            name='attachment',
            field=models.FileField(null=True, upload_to='invoices/%Y/%m/%d', verbose_name='Attachment'),
        ),
        migrations.AlterField(
            model_name='email',
            name='bcc',
            field=models.TextField(null=True, verbose_name='bcc'),
        ),
        migrations.AlterField(
            model_name='email',
            name='body',
            field=models.TextField(verbose_name='Body'),
        ),
        migrations.AlterField(
            model_name='email',
            name='cc',
            field=models.TextField(null=True, verbose_name='cc'),
        ),
        migrations.AlterField(
            model_name='email',
            name='html_body',
            field=models.TextField(verbose_name='HTML Body'),
        ),
        migrations.AlterField(
            model_name='email',
            name='mail_to',
            field=models.EmailField(max_length=254, verbose_name='Mail To'),
        ),
        migrations.AlterField(
            model_name='email',
            name='sent_at',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Sent At'),
        ),
        migrations.AlterField(
            model_name='email',
            name='start_send_at',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Start Send At'),
        ),
        migrations.AlterField(
            model_name='email',
            name='subject',
            field=models.CharField(max_length=255, verbose_name='Subject'),
        ),
    ]



import pytest

from django.conf import settings
from django.core import mail
from django.core.management import call_command

from mailing.models import Email
from mailing.services import process_whitelist, send_mail
from mailing.tasks import send_mail_task

from tests.mailing_factories import EmailFactory


@pytest.fixture(autouse=True)
def reset_email_whitelist():
    """Reset email_whitelist setting before and after each test."""
    settings.EMAIL_WHITELIST = []
    yield
    settings.EMAIL_WHITELIST = []


@pytest.mark.django_db
def test_send_email():
    """Test email creation and sending."""
    assert Email.objects.count() == 0
    subject = 'Test'
    mail_to = 'recipient@example.com'
    body = 'Hello'
    email = EmailFactory(
        subject=subject,
        mail_to=mail_to,
        body=body,
    )
    assert Email.objects.count() == 1
    send_mail(email)
    assert len(mail.outbox) == 1
    assert mail.outbox[0].subject == 'Test'
    Email.objects.queue_email(subject, body, mail_to, body)
    assert len(mail.outbox) == 2


@pytest.mark.django_db
def test_send_email_when_blocked():
    """Test send_email handles whitelist processing."""
    settings.EMAIL_WHITELIST = ['*@portant.shop']
    mail_to = 'vedran@pinkdroids.com'
    cc = 'vedran@example.com'
    bcc = 'bcc@example.com'
    subject = 'subject'
    email = EmailFactory(
        mail_to=mail_to,
        cc=cc,
        bcc=bcc,
        subject=subject
    )
    email_message = send_mail(email)
    assert email_message.subject == 'Blocked'
    assert email_message.body == 'Blocked'
    assert len(mail.outbox) == 0
    processed = Email.objects.get(pk=email.pk)
    assert processed.subject == '[TEST EMAIL] subject'


@pytest.mark.django_db
def test_send_email_task():
    """Test the shared task function that handles the outgoing emails."""
    email = Email.objects.create(
        subject='Test',
        mail_to='recipient@example.com',
        body='Hello',
    )
    assert Email.objects.count() == 1
    send_mail_task.delay(email.pk)
    assert len(mail.outbox) == 1


@pytest.mark.django_db
def test_sendmail_command():
    """Test the sendmail management command."""
    Email.objects.create(
        subject='Test',
        mail_to='recipient@example.com',
        body='Hello',
    )
    call_command('sendmail')
    assert len(mail.outbox) == 1


@pytest.mark.django_db
def test_whitelist():
    """Test email_whitelist functionality."""
    mail_to = 'vedran@pinkdroids.com'
    cc = 'vedran@example.com'
    bcc = 'bcc@example.com'
    subject = 'subject'
    email = EmailFactory(
        mail_to=mail_to,
        cc=cc,
        bcc=bcc,
        subject=subject
    )

    processed = process_whitelist(email)
    assert len(settings.EMAIL_WHITELIST) == 0
    assert processed.mail_to == mail_to
    assert processed.cc == cc
    assert processed.bcc == bcc
    assert processed.subject == subject
    assert processed.start_send_at is None
    assert processed.sent_at is None

    settings.EMAIL_WHITELIST = ['*@pinkdroids.com']
    email = EmailFactory(
        mail_to=mail_to,
        cc=cc,
        bcc=bcc,
        subject=subject
    )
    processed = process_whitelist(email)
    assert processed.mail_to == mail_to
    assert processed.cc is None
    assert processed.bcc is None
    assert processed.subject == f'[TEST EMAIL] {subject}'
    assert processed.start_send_at is None
    assert processed.sent_at is None

    settings.EMAIL_WHITELIST = ['*@example.com']
    email = EmailFactory(
        mail_to=mail_to,
        cc=cc,
        bcc=bcc,
        subject=subject
    )
    processed = process_whitelist(email)
    assert processed.mail_to == cc
    assert processed.cc == cc
    assert processed.bcc == bcc
    assert processed.subject == f'[TEST EMAIL] {subject}'
    assert processed.start_send_at is None
    assert processed.sent_at is None

    settings.EMAIL_WHITELIST = ['info@pinkdroids.com']
    email = EmailFactory(
        mail_to=mail_to,
        cc=cc,
        bcc=bcc,
        subject=subject
    )
    processed = process_whitelist(email)
    assert processed.mail_to == 'vedranATpinkdroids.com'
    assert processed.cc == 'vedranATexample.com'
    assert processed.bcc == 'bccATexample.com'
    assert processed.subject == f'[TEST EMAIL] {subject}'
    assert processed.start_send_at is not None
    assert processed.sent_at is not None

    settings.EMAIL_WHITELIST = [mail_to]
    email = EmailFactory(
        mail_to=mail_to,
        cc=cc,
        bcc=bcc,
        subject=subject
    )
    processed = process_whitelist(email)
    assert processed.mail_to == mail_to
    assert processed.cc is None
    assert processed.bcc is None
    assert processed.subject == f'[TEST EMAIL] {subject}'
    assert processed.start_send_at is None
    assert processed.sent_at is None

    settings.EMAIL_WHITELIST = ['*@example.com']
    email = EmailFactory(
        mail_to=mail_to,
        cc=None,
        bcc=None,
        subject=subject
    )
    processed = process_whitelist(email)
    assert processed.mail_to == 'vedranATpinkdroids.com'
    assert processed.cc is None
    assert processed.bcc is None
    assert processed.subject == f'[TEST EMAIL] {subject}'
    assert processed.start_send_at is not None
    assert processed.sent_at is not None



from appconf import AppConf

from django.conf import settings  # noqa


def resolve(setting, **kwargs):
    """Resolve setting to a result of a callable or itself."""
    if hasattr(setting, '__call__'):
        return setting(**kwargs)
    return setting


class WSPayAppConf(AppConf):

    EMAIL_FROM = ''
    SEND_TASK = 'mailing.tasks.send_mail_task'

    class Meta:
        prefix = 'mailing'
        required = []

    def configure_send_task(self, value):
        if hasattr(value, '__call__'):
            return value

        # Try resolving a setting to a fully qualified name
        # of a function, return the function object if found
        try:
            modname, part_symbol, attr = value.rpartition('.')
            assert part_symbol == '.', value
            assert modname != '', value
        except Exception:
            return value

        try:
            m = __import__(modname, fromlist=[attr])
            f = getattr(m, attr)
            return f
        except Exception:
            raise



from setuptools import find_packages, setup


NAME = "portant-mailing"
DESCRIPTION = "a Django mailing app for Portant shop"
AUTHOR = "Vedran Vojvoda"
AUTHOR_EMAIL = "vedran@pinkdroids.com"
URL = "https://github.com/portant-shop/mailing"
LONG_DESCRIPTION = """
============
Django WSPay
============

This django app provides simple support for payments using WSPay gateway.
"""

tests_require = [
    "pytest",
    "pytest-django"
]

setup(
    name=NAME,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/x-rst',
    version="1.0.2",
    license="MIT",
    url=URL,
    packages=find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Framework :: Django",
    ],
    include_package_data=True,
    install_requires=[
        "celery>=5.2.3",
        "Django>=3.0",
        "redis>=4.1.4",
        "django-appconf"
    ],
    extras_require={
        "testing": tests_require,
    },
    zip_safe=False,
)
